classdef PiecewiseContinuous < FunctionSystem
    
    properties(SetAccess = private)
        P
        T
        NumCells
        DimPerCell
    end
    
    methods
        function obj = PiecewiseContinuous(P, T)
            if ~isa(P, 'LagrangeSystem')
                error('The argument P must be a Lagrange system.');
            end
            if ~isa(T, 'IntervalTriangulation')
                error('The argument T must be an Interval Triangulation.');
            end
            
            obj.P = P;
            obj.T = T;
            
            obj.NumCells = size(T.edges, 1);
            obj.DimPerCell = P.N;
            
            obj.N = obj.NumCells * obj.DimPerCell;
        end
        
        function y = evaluate(obj, x)
            if ~isscalar(x)
                error('The argument must be a scalar.');
            end
            
            y = obj.evaluateAll(x);
        end
        
        function y = evaluateAll(obj, x)
            if ~isvector(x)
                error('The argument x must be a vector.');
            end
            
            nT = zeros(size(x));
            
            rowInd = [];
            colInd = [];
            val = [];
            
            for I = 1:obj.NumCells
                a = obj.T.points(obj.T.edges(I,1));
                b = obj.T.points(obj.T.edges(I,2));
                
                K = find((x >= a) & (x <= b));
                if ~isempty(K)
                    nT(K) = nT(K)+1;
                    
                    pt = (1/(b-a)).*(x(K)-a);
                    
                    offset = (I-1)*obj.DimPerCell;
                    
                    i = offset+repmat(1:obj.DimPerCell, length(K), 1);
                    j = repmat(K', obj.DimPerCell, 1);
                    v = obj.P.evaluateAll(pt)';
                    
                    rowInd = [rowInd; i(:)];
                    colInd = [colInd; j(:)];
                    val = [val; v(:)];
                end
            end
            
            n = obj.NumCells*obj.DimPerCell;
            m = length(x);
            y = sparse(rowInd, colInd, val, n, m)*spdiags((1./nT)', 0, m, m);
        end
        
        function f = get(obj, i)
            if ~isscalar(i) || (i < 1) || (i > obj.N)
                error('The argument N is not a suitable index.');
            end
            
            e = zeros(1, obj.N);
            e(i) = 1;
            f = FunSysFunction(obj, e);
        end
    end
    
end

